@@ -7,6 +7,13 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE RankNTypes #-}
+-- Can only mark as Safe when using a newer GHC, otherwise we get build
+-- failures due to the manual Typeable instance below.
+#if __GLASGOW_HASKELL__ >= 707
+{-# LANGUAGE Safe #-}
+#else
+{-# LANGUAGE Trustworthy #-}
+#endif
 
 module Control.Monad.Trans.Resource.Internal(
     InvalidAccess(..)
@@ -23,7 +30,8 @@ module Control.Monad.Trans.Resource.Internal(
 ) where
 
 import Control.Exception (throw,Exception,SomeException)
-import Control.Applicative (Applicative (..))
+import Control.Applicative (Applicative (..), Alternative(..))
+import Control.Monad (MonadPlus(..))
 import Control.Monad.Trans.Control
     ( MonadTransControl (..), MonadBaseControl (..) )
 import Control.Monad.Base (MonadBase, liftBase)
@@ -54,7 +62,6 @@ import Control.Monad.IO.Class (MonadIO (..))
 import Control.Monad (liftM)
 #endif
 import qualified Control.Exception as E
-import Control.Monad.ST (ST)
 import Control.Monad.Catch (MonadThrow (..), MonadCatch (..)
 #if MIN_VERSION_exceptions(0,6,0)
     , MonadMask (..)
@@ -69,20 +76,6 @@ import Data.Word(Word)
 import Prelude hiding (catch)
 import Data.Acquire.Internal (ReleaseType (..))
 
-#if __GLASGOW_HASKELL__ >= 704
-import Control.Monad.ST.Unsafe (unsafeIOToST)
-#else
-import Control.Monad.ST (unsafeIOToST)
-#endif
-
-#if __GLASGOW_HASKELL__ >= 704
-import qualified Control.Monad.ST.Lazy.Unsafe as LazyUnsafe
-#else
-import qualified Control.Monad.ST.Lazy as LazyUnsafe
-#endif
-
-import qualified Control.Monad.ST.Lazy as Lazy
-
 import Control.Monad.Morph
 
 -- | A @Monad@ which allows for safe resource allocation. In theory, any monad
@@ -235,8 +228,20 @@ instance Applicative m => Applicative (ResourceT m) where
     ResourceT mf <*> ResourceT ma = ResourceT $ \r ->
         mf r <*> ma r
 
+-- | Since 1.1.5
+instance Alternative m => Alternative (ResourceT m) where
+    empty = ResourceT $ \_ -> empty
+    (ResourceT mf) <|> (ResourceT ma) = ResourceT $ \r -> mf r <|> ma r
+
+-- | Since 1.1.5
+instance MonadPlus m => MonadPlus (ResourceT m) where
+    mzero = ResourceT $ \_ -> mzero
+    (ResourceT mf) `mplus` (ResourceT ma) = ResourceT $ \r -> mf r `mplus` ma r
+
 instance Monad m => Monad (ResourceT m) where
+#if !MIN_VERSION_base(4,8,0)
     return = ResourceT . const . return
+#endif
     ResourceT ma >>= f = ResourceT $ \r -> do
         a <- ma r
         let ResourceT f' = f a
